# coding=utf-8
from __future__ import absolute_import, division, print_function

__metaclass__ = type

from astwro.phot import iphas

# def test_iphas():
#     sct = iphas.Simulated_ri_rHa_Colors(reddening=0.44)
#     sct.r_Ha
