from __future__ import absolute_import, division, print_function

import logging

# Override to your logger for logging, e.g:
#      pydaophot.logger = logging.getLogger('root').getChild('pydaophot')
logger = logging.getLogger('astwro')
# logger.addHandler(logging.NullHandler())
